package com.pjb.springbootjjwt.interceptor;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.pjb.springbootjjwt.entity.Result;
import com.pjb.springbootjjwt.exception.SoaException;
import com.pjb.springbootjjwt.exception.SuccessException;
import com.pjb.springbootjjwt.utils.ResultUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Slf4j
@ControllerAdvice
public class GloablExceptionHandler {

    @ResponseBody
    @ExceptionHandler(Exception.class)
    public Object handleException(Exception e) {
        String msg = e.getMessage();
        if (msg == null || msg.equals("")) {
            msg = "服务器出错";
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("message", msg);
        return ResultUtils.errorResult(msg);
//        return jsonObject;
    }

    @ResponseBody
    @ExceptionHandler(SoaException.class)
    public Result handleSoaException(Exception e) {
        String msg = e.getMessage();
        if (msg == null || msg.equals("")) {
            msg = "服务器出错";
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("message", msg);
        return ResultUtils.successResult(msg);
    }

    @ResponseBody
    @ExceptionHandler(SuccessException.class)
    public Result handleSucException(Exception e) {
        String msg = e.getMessage();
        if (msg == null || msg.equals("")) {
            msg = "服务器出错";
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("message", msg);
        return ResultUtils.errorResult(msg);
    }


    /**
     * @description: validation Exception
     * @param: [exception]
     * @return: com.pjb.springbootjjwt.entity.Result
     * @author: Jiay
     * @date: 2022-5-25 11:23
     */
    @ExceptionHandler({ MethodArgumentNotValidException.class })
    @ResponseBody
    public Result handleBodyValidException(MethodArgumentNotValidException exception) {
        String argNotValidMessage = getArgNotValidMessage(exception.getBindingResult());
        log.error("MethodArgumentNotValidException   " + argNotValidMessage);
        return ResultUtils.errorResult(argNotValidMessage);
    }

    /**
     * @description: 获取请求参数不正确的提示信息,多个信息，拼接成用逗号分隔的形式
     * @param: [bindingResult]
     * @return: java.lang.String
     * @author: Jiay
     * @date: 2022-5-25 11:22
     */
    private String getArgNotValidMessage(BindingResult bindingResult) {
        if (bindingResult == null) {
            return "";
        }
        StringBuilder stringBuilder = new StringBuilder();

        //多个错误用逗号分隔
        List<ObjectError> allErrorInfos = bindingResult.getAllErrors();
        for (ObjectError error : allErrorInfos) {
            stringBuilder.append(",").append(error.getDefaultMessage());
        }

        //最终把首部的逗号去掉
        return StrUtil.removePrefix(stringBuilder.toString(), ",");
    }

}
