/*============================================================================================================
File description:

Filename: ProductDrawingServiceImpl.java
Module  :
	KHom006-MES集成：图纸齐套性接口
	MES调用TC接口，根据提供的物料号，TC返回物料号对应的产品图纸和工艺图纸信息
============================================================================================================
DATE           Name             Description of Change
2024-04-12     liujb		    Creation

$HISTORY$
2024-04-12     liujb          Modify：
============================================================================================================*/
package com.pjb.springbootjjwt.service.impl;

import com.alibaba.fastjson.JSON;
import com.pjb.springbootjjwt.entity.CompletenessInfo;
import com.pjb.springbootjjwt.entity.ProductDrawingInfo;
import com.pjb.springbootjjwt.entity.ProductDrawingInput;
import com.pjb.springbootjjwt.entity.Result;
import com.pjb.springbootjjwt.entity.input.CompletenessInput;
import com.pjb.springbootjjwt.mapper.MesMapper;
import com.pjb.springbootjjwt.service.MesService;
import com.pjb.springbootjjwt.utils.LogUtil;
import com.pjb.springbootjjwt.utils.ResultUtils;
import com.pjb.springbootjjwt.utils.TCUtils;
import com.teamcenter.clientx.AppXSession;
import com.teamcenter.clientx.Query;
import com.teamcenter.soa.client.model.ModelObject;
import com.teamcenter.soa.client.model.Property;
import com.teamcenter.soa.client.model.strong.Item;
import com.teamcenter.soa.client.model.strong.ItemRevision;
import com.teamcenter.soa.client.model.strong.MEProcessRevision;
import com.teamcenter.soa.exceptions.NotLoadedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class MesServiceImpl implements MesService {
    @Autowired
    MesMapper productDrawingMapper;
    private final static String KH_FILTER_STATUS = "KH_Filter_Status"; //过滤状态首选项

    /**
     * KHom005-MES集成：产品图纸接口
     * MES调用TC接口，根据提供的入参信息，TC返回对应图纸信息及附件
     */
    @Override
    public Result getProductDrawingInfo(ProductDrawingInput productDrawingInput) {
        Result result = new Result();
        String log_result[] = {""};
        AppXSession session = null;
        try {
            TCUtils.login();
            session = TCUtils.getSession();
            TCUtils.setByPass(session, true);

            String partID = productDrawingInput.getPartID();
            String gxNO = productDrawingInput.getGXNo();
            String drawingRev = productDrawingInput.getDrawingRev();
            if(!partID.isEmpty()){
                if( !gxNO.isEmpty()) {
                    if(!drawingRev.isEmpty()){
                        result = getGyDrawing(session,partID,gxNO,result,log_result,drawingRev);
                    }else{
                        result = getGyDrawing(session,partID,gxNO,result,log_result,null);
                    }

                }else{
                    System.out.println("Has gxNO");
                    if(!drawingRev.isEmpty()){
                        result = getProductDrawing(session,partID,result,log_result,drawingRev);
                    }else{
                        result = getProductDrawing(session,partID,result,log_result,null);
                    }

                }

            }


        }catch (Exception e) {
            e.printStackTrace();
            LogUtil.error(MesServiceImpl.class, e);
            return ResultUtils.errorResult(e);
        } finally {
            try {
                TCUtils.setByPass(session, false);
            } catch (Exception e) {
                e.printStackTrace();
                LogUtil.error(MesServiceImpl.class, e);
            }
            System.out.println("log_result[0]:"+log_result[0]);
            productDrawingMapper.insertKH_LOG("MES002","产品图纸接口","PLM","MES","","",log_result[0],JSON.toJSONString(productDrawingInput),JSON.toJSONString(result),new SimpleDateFormat("yyyy/MM/dd hh:mm:ss").format(new Date()));
            TCUtils.logoutUser(session);
        }
        return result;
    }

    /**
     * 获取工艺图纸
     * @param session
     * @param partID
     * @param result
     * @param log_result
     * @return
     */
    private Result getGyDrawing(AppXSession session, String partID,String gxNO, Result result, String[] log_result, String drawingRevId) throws Exception {
        String[] Key = {"零组件 ID"};
//            String[] Key = {"Item ID"};
        String[] Values = {partID};
//            ModelObject[] modelObjects = Query.queryItems("零组件...", Key, Values);
        ModelObject[] modelObjects = Query.queryItems("Item...", Key, Values);
        if(modelObjects == null || modelObjects.length == 0){
            //存在第一次查询为空的情况
            modelObjects = Query.queryItems("Item...", Key, Values);
            if(modelObjects == null || modelObjects.length == 0){
                result = ResultUtils.errorResult("物料不存在");
                log_result[0] = "失败";
                return result;
            }
        }

        //过滤发布状态
        String[] preferences = TCUtils.getPreferenceValue(session, KH_FILTER_STATUS);

        ItemRevision partItemRev = null;
//        if(drawingRev!=null){
//            partItemRev = TCUtils.getRevisionReleasedByRevID(session, modelObjects[0],drawingRev);
//        }else{

            partItemRev = TCUtils.getRevisionReleased_Filter(session, modelObjects[0],preferences);

//        }


        if(partItemRev == null){
            result = ResultUtils.errorResult("此物料最新版本未发布，无法提供图纸信息");
            log_result[0] = "失败";
            return  result;
        }else{
            ProductDrawingInfo info = new ProductDrawingInfo();
            TCUtils.setModelObjectValue(session, new ModelObject[]{partItemRev}, new String[]{"item_id", "item_revision_id" ,"TC_Is_Represented_By"});
            String partItemRevID = partItemRev.getPropertyDisplayableValue("item_revision_id");
            String partItemID = partItemRev.getPropertyDisplayableValue("item_id");
            info.setPartID(partItemID);
            info.setPartRevID(partItemRevID);



            //获取工艺
            List<ModelObject> meProcessRevisions = TCUtils.getWhereReference(session, partItemRev, "IMAN_METarget", "MEProcessRevision", 1);
            System.out.println("meProcessRevisions.size():"+meProcessRevisions.size());
            if(meProcessRevisions != null && meProcessRevisions.size() != 0 ){
                MEProcessRevision meProcessRevision = (MEProcessRevision) meProcessRevisions.get(0);
                Item items_tag = meProcessRevision.get_items_tag();
                ItemRevision meProcessRevisionReleased = TCUtils.getRevisionReleased(session, items_tag);
//                    ModelObject[] bomLineByWork = TCUtils.getBomLineByWork(session, meProcessRevisionReleased);
                ModelObject[] bomLines = TCUtils.getBomLine(session, meProcessRevisionReleased);
                System.out.println("bomLines.size():"+bomLines.length);
                if(bomLines != null &&bomLines.length != 0){
                    boolean existGxNO = false;
                    for(int i =0; i < bomLines.length ; i++){
                        TCUtils.setModelObjectValue(session,bomLines[i],"mfg0realRevision");
                        Property mfg0realRevision = bomLines[i].getPropertyObject("mfg0realRevision");
                        ModelObject modelObjectValue = mfg0realRevision.getModelObjectValue();
                        TCUtils.setModelObjectValue(session,modelObjectValue,"IMAN_reference");
                        ModelObject[] iman_references = modelObjectValue.getPropertyObject("IMAN_reference").getModelObjectArrayValue();
                        System.out.println("iman_references.size():"+iman_references.length);
                        if(iman_references != null && iman_references.length != 0){
                            ItemRevision revisionReleased_filter = TCUtils.getRevisionReleased_Filter(session, iman_references[0], preferences);


                            Item drawingItem = ((ItemRevision) revisionReleased_filter).get_items_tag();
                            ItemRevision drawningItemRev = null;
                            if(drawingRevId != null){
                                drawningItemRev = TCUtils.getRevisionReleasedByRevID(session, drawingItem,drawingRevId);
                            }
                            else{

                                drawningItemRev = (ItemRevision) revisionReleased_filter;
                            }
                            if(drawningItemRev == null){
                                result = ResultUtils.errorResult("此物料最新版本未发布，无法提供图纸信息");
                                log_result[0] = "失败";
                                return  result;
                            }

                            TCUtils.setModelObjectValue(session,drawningItemRev,new String[]{"item_id","item_revision_id","IMAN_specification"});
                            String gxtz_item_id = drawningItemRev.getPropertyDisplayableValue("item_id");
                            String gxtz_item_revision_id  = drawningItemRev.getPropertyDisplayableValue("item_revision_id");
                            String gxseq = bomLines[i].getPropertyDisplayableValue("bl_sequence_no");
                            System.out.println("gxtz_item_id:"+gxtz_item_id);
                            System.out.println("gxtz_item_revision_id:"+gxtz_item_revision_id);
                            System.out.println("gxseq:"+gxseq);
                            if(gxNO.contains(gxseq)){
                                existGxNO = true;
                                List<String> filePathList = TCUtils.getFileByFMS(session, drawningItemRev, "IMAN_specification",gxtz_item_id , gxtz_item_revision_id);
                                String filePathString  = "";
                                for(String filePath : filePathList){
                                    if(filePathString.length() != 0){
                                        filePathString += "|";
                                    }
                                    filePathString += filePath;
                                }

                                info.setDrawingID(gxtz_item_id);
                                info.setDrawingRev(gxtz_item_revision_id);
                                info.setFile(filePathString);
                                info.setFile2("");
                                info.setFile3("");
                                result.setData(JSON.toJSON(info));

                                result = ResultUtils.successResult(info);
                                log_result[0]= "成功";
                            }
                        }
                    }

                    if(!existGxNO){

                        result = ResultUtils.errorResult("此物料无工序图纸");
                        log_result[0] = "失败";
                    }
                }
            }

        }
        return  result;
    }

    /**
     * 获取产品图纸
     * @param session
     * @param partID
     * @param result
     * @param log_result
     * @return
     * @throws Exception
     */
    private Result getProductDrawing(AppXSession session,String partID,Result result,String log_result[],String drawingRevId) throws Exception {
        String[] Key = {"零组件 ID"};
//            String[] Key = {"Item ID"};
        String[] Values = {partID};
//            ModelObject[] modelObjects = Query.queryItems("零组件...", Key, Values);

        ModelObject[] modelObjects = Query.queryItems("Item...", Key, Values);
        if(modelObjects == null || modelObjects.length == 0){
            //存在第一次查询为空的情况
            modelObjects = Query.queryItems("Item...", Key, Values);
            if(modelObjects == null || modelObjects.length == 0){
                result = ResultUtils.errorResult("物料不存在");
                log_result[0] = "失败";
                return result;
            }
        }
        ItemRevision partItemRev = null;
//        if(drawingRev != null){
//            partItemRev = TCUtils.getRevisionReleasedByRevID(session, modelObjects[0],drawingRev);
//        }
//        else{
            //过滤发布状态
            String[] preferences = TCUtils.getPreferenceValue(session, KH_FILTER_STATUS);
            partItemRev = TCUtils.getRevisionReleased_Filter(session, modelObjects[0],preferences);
//        }



        if(partItemRev == null){
            result = ResultUtils.errorResult("此物料最新版本未发布，无法提供图纸信息");
            log_result[0] = "失败";
            return  result;
        }else{
            ProductDrawingInfo info = new ProductDrawingInfo();
            TCUtils.setModelObjectValue(session, new ModelObject[]{partItemRev}, new String[]{"item_id", "item_revision_id" ,"TC_Is_Represented_By"});
            String partItemRevID = partItemRev.getPropertyDisplayableValue("item_revision_id");
            String partItemID = partItemRev.getPropertyDisplayableValue("item_id");
            info.setPartID(partItemID);
            info.setPartRevID(partItemRevID);

            Property tc_is_represented_by = partItemRev.getPropertyObject("TC_Is_Represented_By");
            ModelObject[] drawningItemRevs = tc_is_represented_by.getModelObjectArrayValue();
            if(drawningItemRevs != null && drawningItemRevs.length != 0 ){
                Item drawingItem = ((ItemRevision) drawningItemRevs[0]).get_items_tag();
                ItemRevision revisionReleased_filter = TCUtils.getRevisionReleased_Filter(session, drawingItem, preferences);
                TCUtils.setModelObjectValue(session, new ModelObject[]{drawningItemRevs[0]}, new String[]{"item_id", "item_revision_id" ,"IMAN_specification","EC_affected_item_rel"});


                ItemRevision drawningItemRev = null;
                if(drawingRevId != null){
                    drawningItemRev = TCUtils.getRevisionReleasedByRevID(session, drawingItem,drawingRevId);
                }
                else{

                    drawningItemRev = (ItemRevision) revisionReleased_filter;
                }
                if(drawningItemRev == null){
                    result = ResultUtils.errorResult("此物料最新版本未发布，无法提供图纸信息");
                    log_result[0] = "失败";
                    return  result;
                }
                String drawingItemID = drawningItemRev.getPropertyDisplayableValue("item_id");
                String drawingItemRevID = drawningItemRev.getPropertyDisplayableValue("item_revision_id");
                List<String> filePathList = TCUtils.getFileByFMS(session, drawningItemRev, "IMAN_specification",drawingItemID , drawingItemRevID);
                String filePathString  = "";
                for(String filePath : filePathList){
                    if(filePathString.length() != 0){
                        filePathString += "|";
                    }
                    filePathString += filePath;
                }

                //获取客户图纸 file2
                String file2PathString  = "";
                List<String> file2PathList = TCUtils.getFolderFileByFMS(session, drawningItemRev, "KH3_CustomerDraw");//KH3_CustomerDraw  EC_affected_item_rel
                for(String file2Path : file2PathList){
                    if(file2PathString.length() != 0){
                        file2PathString += "|";
                    }
                    file2PathString += file2Path;
                }
//                    ModelObject[] customerDrawnings = drawningItemRevs[0].getPropertyObject("EC_affected_item_rel").getModelObjectArrayValue();
//                    if(customerDrawnings != null && customerDrawnings.length != 0){
//                        TCUtils.setModelObjectValue(session, customerDrawnings, new String[]{"object_type"});
//                        for(ModelObject customerDrawning : customerDrawnings){
//                            if(customerDrawning.getPropertyDisplayableValue("object_type").endsWith("PDF")){
//                                System.out.println("true");
//                            }else{
//
//                                System.out.println("false");
//                            }
//                        }
//                    }
                //获取外发图 flie3
                String file3PathString  = "";
//                    String twDrawingItemId = drawingItemID.replace("TC","TW");
                ModelObject[] twDrawings = Query.queryItems("外发图纸对象查询V1.0", new String[]{"设计图ID"}, new String[]{drawingItemID});
                if(twDrawings != null && twDrawings.length != 0){
                    for(ModelObject twDrawing : twDrawings) {
                        ItemRevision twDrawingRev = TCUtils.getRevisionReleased(session, twDrawing);
                        if(twDrawingRev != null){
                            TCUtils.setModelObjectValue(session, new ModelObject[]{twDrawingRev}, new String[]{"item_id", "item_revision_id" ,"IMAN_specification"});
                            String twDrawingItemID = twDrawingRev.getPropertyDisplayableValue("item_id");
                            String twDrawingItemRevID = twDrawingRev.getPropertyDisplayableValue("item_revision_id");
                            List<String> file3PathList = TCUtils.getFileByFMS(session, twDrawingRev, "IMAN_specification",twDrawingItemID , twDrawingItemRevID);

                            for(String file3Path : file3PathList){
                                if(file3PathString.length() != 0){
                                    file3PathString += "|";
                                }
                                file3PathString += file3Path;
                            }
                        }
                    }
                }
                info.setDrawingID(drawingItemID);
                info.setDrawingRev(drawingItemRevID);
                info.setFile(filePathString);
                info.setFile2(file2PathString);
                info.setFile3(file3PathString);
                result.setData(JSON.toJSON(info));

            }
            result = ResultUtils.successResult(info);
            log_result[0]= "成功";
        }
        return  result;
    }

    /**
     * KHom006-MES集成：图纸齐套性接口
     * MES调用TC接口，根据提供的物料号，TC返回物料号对应的产品图纸和工艺图纸信息
     */
    @Override
    public Result getCompletenessInfo(CompletenessInput completenessInput) {
        Result result = new Result();
        String log_result = "";
        AppXSession session = null;
        try {
            TCUtils.login();
            session = TCUtils.getSession();
            TCUtils.setByPass(session, true);
            List<ModelObject> allMoList = new ArrayList<>();
            List<String> partIds = completenessInput.getPartID();
            for(String partId :partIds ){
                String[] Key = {"零组件 ID"};
//            String[] Key = {"Item ID"};
                String[] Values = {partId};
//            ModelObject[] modelObjects = Query.queryItems("零组件...", Key, Values);
                ModelObject[] modelObjects = Query.queryItems("Item...", Key, Values);
                List<ModelObject> moList= Arrays.asList(modelObjects);
                allMoList.addAll(moList);

            }
            if(allMoList == null || allMoList.size() == 0){
                result = ResultUtils.errorResult("物料不存在");
                log_result = "失败";
                return  result;
            }
            //正常物料
            List<ModelObject>  normalMoList = new ArrayList<>();
            //异常物料
            List<ModelObject>  unnormalMoList = new ArrayList<>();
            //过滤发布状态
//          String[] preferences = {"TC 基线"};
            String[] preferences = TCUtils.getPreferenceValue(session, KH_FILTER_STATUS);
            for(ModelObject mo : allMoList){
                ItemRevision partItemRev = TCUtils.getRevisionReleased_Filter(session, mo, preferences);
                TCUtils.setModelObjectValue(session, new ModelObject[]{partItemRev}, new String[]{"item_id", "item_revision_id" ,"TC_Is_Represented_By"});
                if(partItemRev == null){
                    unnormalMoList.add(mo);
//                    return  result;
                }else{
                    normalMoList.add(partItemRev);

                }


            }
            List<CompletenessInfo> infoList = new ArrayList<CompletenessInfo>();

            if(unnormalMoList.size() != 0 ){
                StringBuilder msgBuilder = new StringBuilder();// result = ResultUtils.errorResult("<"+partItemID+">此物料最新版本未发布，无法提供图纸信息");
                for(ModelObject unnormalMo : unnormalMoList){
                    if(msgBuilder.length() != 0 ){
                        msgBuilder.append(",");
                    }
                    TCUtils.setModelObjectValue(session, new ModelObject[]{unnormalMo}, new String[]{"item_id"});
                    String partItemID = unnormalMo.getPropertyDisplayableValue("item_id");
                    msgBuilder.append("<");
                    msgBuilder.append(partItemID);
                    msgBuilder.append(">");
                }
                msgBuilder.append("物料最新版本未发布，无法提供图纸信息");
                log_result = "失败";
                result = ResultUtils.errorResult(msgBuilder.toString());
                return result;
            }else{
                for(ModelObject partItemRev : normalMoList){

                    CompletenessInfo productDrawinginfo = new CompletenessInfo();

                    String partItemRevID = partItemRev.getPropertyDisplayableValue("item_revision_id");
                    String partItemID = partItemRev.getPropertyDisplayableValue("item_id");
                    productDrawinginfo.setPartID(partItemID);
                    productDrawinginfo.setPartRevID(partItemRevID);

                    Property tc_is_represented_by = partItemRev.getPropertyObject("TC_Is_Represented_By");
                    ModelObject[] drawningItemRevs = tc_is_represented_by.getModelObjectArrayValue();
                    //制造目标下的图纸
                    if(drawningItemRevs != null && drawningItemRevs.length != 0 ){
                        TCUtils.setModelObjectValue(session, new ModelObject[]{drawningItemRevs[0]}, new String[]{"item_id", "item_revision_id" ,"IMAN_specification"});
                        String drawingItemID = drawningItemRevs[0].getPropertyDisplayableValue("item_id");
                        String drawingItemRevID = drawningItemRevs[0].getPropertyDisplayableValue("item_revision_id");

                        productDrawinginfo.setDrawingID(drawingItemID);
                        productDrawinginfo.setDrawingRev(drawingItemRevID);
                        infoList.add(productDrawinginfo);

                    }
                    //获取工艺
                    List<ModelObject> meProcessRevisions = TCUtils.getWhereReference(session, partItemRev, "IMAN_METarget", "MEProcessRevision", 1);
                    if(meProcessRevisions != null && meProcessRevisions.size() != 0 ){
                        MEProcessRevision meProcessRevision = (MEProcessRevision) meProcessRevisions.get(0);
                        Item items_tag = meProcessRevision.get_items_tag();
                        ItemRevision meProcessRevisionReleased = TCUtils.getRevisionReleased(session, items_tag);
//                    ModelObject[] bomLineByWork = TCUtils.getBomLineByWork(session, meProcessRevisionReleased);
                        ModelObject[] bomLines = TCUtils.getBomLine(session, meProcessRevisionReleased);
                        if(bomLines != null &&bomLines.length != 0){
                            for(int i =0; i < bomLines.length ; i++){

                                TCUtils.setModelObjectValue(session,bomLines[i],"mfg0realRevision");
                                Property mfg0realRevision = bomLines[i].getPropertyObject("mfg0realRevision");
                                ModelObject modelObjectValue = mfg0realRevision.getModelObjectValue();
                                TCUtils.setModelObjectValue(session,modelObjectValue,"IMAN_reference");
                                ModelObject[] iman_references = modelObjectValue.getPropertyObject("IMAN_reference").getModelObjectArrayValue();
                                if(iman_references != null && iman_references.length != 0){
                                    ItemRevision revisionReleased_filter = TCUtils.getRevisionReleased_Filter(session, iman_references[0], preferences);
                                    if(revisionReleased_filter == null){
                                        break;
                                    }
                                    CompletenessInfo gxDrawinginfo = new CompletenessInfo();
                                    gxDrawinginfo.setPartID(partItemID);
                                    gxDrawinginfo.setPartRevID(partItemRevID);

                                    TCUtils.setModelObjectValue(session,revisionReleased_filter,new String[]{"item_id","item_revision_id"});
                                    String gxtz_item_id = revisionReleased_filter.getPropertyDisplayableValue("item_id");
                                    String gxtz_item_revision_id  = revisionReleased_filter.getPropertyDisplayableValue("item_revision_id");
                                    String gxno = bomLines[i].getPropertyDisplayableValue("bl_sequence_no");
                                    gxDrawinginfo.setDrawingID(gxtz_item_id);
                                    gxDrawinginfo.setDrawingRev(gxtz_item_revision_id);
                                    gxDrawinginfo.setGXno(gxno);
                                    infoList.add(gxDrawinginfo);
                                    System.out.println();
                                }
                            }
                        }
                    }
                }

                if(infoList.size() != 0){
                    result = ResultUtils.successResult(infoList);
                    log_result = "成功";
                }else{
                    result = ResultUtils.errorResult("此物料产品图纸和工序图纸都不存在");
                    log_result = "失败";
                }

            }



        }catch (Exception e) {
            e.printStackTrace();
            LogUtil.error(MesServiceImpl.class, e);
            return ResultUtils.errorResult(e);
        } finally {
            try {
                TCUtils.setByPass(session, false);
            } catch (Exception e) {
                e.printStackTrace();
                LogUtil.error(MesServiceImpl.class, e);
            }
            productDrawingMapper.insertKH_LOG("MES004","产品图纸信息接口","PLM","MES","","",log_result,JSON.toJSONString(completenessInput),JSON.toJSONString(result),new SimpleDateFormat("yyyy/MM/dd hh:mm:ss").format(new Date()));
            TCUtils.logoutUser(session);
        }
        return result;
    }

    public ProductDrawingInfo getProductDrawingInfoByPartID() {
        return null;
    }


}
