package com.pjb.springbootjjwt.service.impl;

import com.alibaba.fastjson.JSON;
import com.pjb.springbootjjwt.SpringbootJjwtApplication;
import com.pjb.springbootjjwt.entity.ImportWFDatasetInput;
import com.pjb.springbootjjwt.entity.ResultOa;
import com.pjb.springbootjjwt.mapper.MesMapper;
import com.pjb.springbootjjwt.service.OaService;
import com.pjb.springbootjjwt.utils.HttpUtils;
import com.pjb.springbootjjwt.utils.LogUtil;
import com.pjb.springbootjjwt.utils.ResultUtils;
import com.pjb.springbootjjwt.utils.TCUtils;
import com.teamcenter.clientx.AppXSession;
import com.teamcenter.services.strong.workflow.WorkflowService;
import com.teamcenter.services.strong.workflow._2014_06.Workflow;
import com.teamcenter.soa.client.model.ModelObject;
import com.teamcenter.soa.client.model.ServiceData;
import com.teamcenter.soa.client.model.strong.Dataset;
import com.teamcenter.soa.client.model.strong.EPMTask;
import com.teamcenter.soa.client.model.strong.ItemRevision;
import com.teamcenter.soa.client.model.strong.POM_system_class;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.stereotype.Service;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class OaServiceImpl implements  OaService {
    @Autowired
    MesMapper productDrawingMapper;

    @Override
    public ResultOa importWFDataset(ImportWFDatasetInput importWFDatasetInput) {
        ResultOa result = new ResultOa();
        String log_result = "";
        //入参验证
        result = validation(importWFDatasetInput);
        if(result.getStatus() == "F"){
            return  result;
        }

        AppXSession session = null;
        try {
            TCUtils.login();
            session = TCUtils.getSession();
            TCUtils.setByPass(session, true);

            ModelObject epmMo= TCUtils.getItemByUID(session,importWFDatasetInput.getWFuid() );//"QPrZ0aRDY8MSRC"
            //是否为流程任务
            if(!(epmMo instanceof EPMTask)){
                log_result = "失败";
                result = ResultUtils.errorResultOa("无效TC流程UID");
                return  result;
            }
            TCUtils.setModelObjectValue(session,epmMo,"root_target_attachments");
            ModelObject[] root_target_attachments = ((EPMTask) epmMo).get_root_target_attachments();
            //目标版本
            ItemRevision targetRev  = null;
            for(ModelObject root_target_attachment : root_target_attachments){
                if(root_target_attachment instanceof ItemRevision){
                    TCUtils.setModelObjectValue(session,root_target_attachment,new String[]{"item_id","item_revision_id"});
                    String item_id = ((ItemRevision) root_target_attachment).get_item_id();
                    String item_revision_id = ((ItemRevision) root_target_attachment).get_item_revision_id();
                    if(importWFDatasetInput.getDocid().equals(item_id) && importWFDatasetInput.getDocrevid().equals(item_revision_id)){
                        targetRev = (ItemRevision) root_target_attachment;
                    }
                }
            }
            if(targetRev == null){
                log_result = "失败";
                result = ResultUtils.errorResultOa("无效目标版本！");
                return  result;
            }
            //获取URL文件

            List<File> files = new ArrayList<File>();
            String urls[] = importWFDatasetInput.getUrl().split("\\|");
            if(urls != null && urls.length != 0) {
                for (String url : urls) {
//                    File file = HttpUtils.downloadFile(url, "D:\\DKFile\\DownLoad");
                    File file = HttpUtils.downloadFile(url, SpringbootJjwtApplication.TC_FCCCACH);
                    if (file != null) {
                        files.add(file);
                    } else {
                        log_result = "失败";
                        result = ResultUtils.errorResultOa("文件下载异常！");
                        return result;
                    }
                }
            }
            //创建数据集附加到版本下
            for(File file : files){
                String fileType = null;
                String[] split = file.getName().split("\\.");
                if("xlsx".equals(split[1])){
                    fileType = "MSExcelX";
                }
                if("xls".equals(split[1])){
                    fileType = "MSExcel";
                }
                if(fileType != null){
                    //目标版本下是否包含附加数据集
                    ModelObject old_tc_attache = null;
                    TCUtils.setModelObjectValue(session,targetRev,"TC_Attaches");
                    ModelObject[] tc_attaches = targetRev.get_TC_Attaches();
                    if(tc_attaches != null && tc_attaches.length != 0){
                        for(ModelObject tc_attache : tc_attaches){
                            if(tc_attache instanceof  Dataset){
                                TCUtils.setModelObjectValue(session,tc_attache,"object_name");
                                String tc_attache_name =  ((Dataset)tc_attache).get_object_name();
                                if(file.getName().equals(tc_attache_name)){
                                   old_tc_attache = tc_attache;
                                }
                            }
                        }
                    }
                    //存在移除关系
                    if(old_tc_attache != null){
                        TCUtils.removeRelation(targetRev,old_tc_attache,"TC_Attaches");
                    }
                    Dataset dataset = TCUtils.createNewDatasetAndUploadSingleFile(session, targetRev, "TC_Attaches", file, file.getName(), fileType, "MSExcel", "excel");
                    if(dataset == null){
                        log_result = "失败";
                        result = ResultUtils.errorResultOa("数据集创建异常！");
                        return  result;
                    }else {
                        //获取版本所有者
                        POM_system_class owning_user = targetRev.get_owning_user();
                        //更改所有权
                        TCUtils.changeOwnershipModelObject(session, owning_user, dataset);
                    }

                }
                else{
                    log_result = "失败";
                    result = ResultUtils.errorResultOa("不支持的文件类型:"+file.getName());
                    return  result;
                }
            }
            //提升流程 fnd0StartedTasks
            EPMTask[] fnd0StartedTasks = ((EPMTask) epmMo).get_fnd0StartedTasks();
            if(fnd0StartedTasks != null && fnd0StartedTasks.length != 0){
                EPMTask fnd0StartedTask = fnd0StartedTasks[0];
                WorkflowService wfService = WorkflowService.getService(session.getConnection());
                Workflow.PerformActionInputInfo[] performActionInputInfos = new Workflow.PerformActionInputInfo[1];
                Workflow.PerformActionInputInfo performActionInputInfo = new Workflow.PerformActionInputInfo();
                performActionInputInfo.action = "SOA_EPM_promote_action";
                performActionInputInfo.actionableObject = fnd0StartedTask;
                performActionInputInfos[0] = performActionInputInfo;
                ServiceData serviceData = wfService.performAction3(performActionInputInfos);
                if(serviceData.sizeOfPlainObjects() != 1){
                    log_result = "失败";
                    result = ResultUtils.errorResultOa("流程提升失败！");
                }else
                {
                    log_result = "成功";
                    result = ResultUtils.successResultOa("成功！");
                }
            }else{
                log_result = "失败";
                result = ResultUtils.errorResultOa("流程提升失败！");
            }



        }catch (Exception e) {
            e.printStackTrace();
            LogUtil.error(MesServiceImpl.class, e);
            log_result = "失败";
            return ResultUtils.errorResultOa(e);
        } finally {
            try {
                TCUtils.setByPass(session, false);
            } catch (Exception e) {
                e.printStackTrace();
                LogUtil.error(MesServiceImpl.class, e);
            }
            productDrawingMapper.insertKH_LOG("OA002","OA-PLM流程接口","PLM","OA","","",log_result, JSON.toJSONString(importWFDatasetInput),JSON.toJSONString(result),new SimpleDateFormat("yyyy/MM/dd hh:mm:ss").format(new Date()));
            TCUtils.logoutUser(session);
            return  result;
        }

    }

    private ResultOa validation(ImportWFDatasetInput importWFDatasetInput) {
        ResultOa  result = new ResultOa();
        StringBuilder errs = new StringBuilder();
        if(importWFDatasetInput == null){
            result = ResultUtils.errorResultOa("无效Body！");
            return result;
        }
        if(importWFDatasetInput.getWFuid() == null || importWFDatasetInput.getWFuid().trim().isEmpty()){
            errs.append("无效TC流程UID");
        }
        if(importWFDatasetInput.getDocid() == null || importWFDatasetInput.getDocid().trim().isEmpty()){
            if(errs.length() != 0){
                errs.append(";");
            }
            errs.append("无效文档ID");
        }
        if(importWFDatasetInput.getDocrevid() == null || importWFDatasetInput.getDocrevid().trim().isEmpty()){
            if(errs.length() != 0){
                errs.append(";");
            }
            errs.append("无效文档版本号");
        }
        if(importWFDatasetInput.getUrl() == null || importWFDatasetInput.getUrl().trim().isEmpty()){
            if(errs.length() != 0){
                errs.append(";");
            }
            errs.append("无效文件链接");
        }
        if(errs.length() != 0){
            result = ResultUtils.errorResultOa(errs.toString());
        }
        return result;
    }
}
