package com.pjb.springbootjjwt.utils;

import java.io.*;

import java.io.File;
import java.net.*;

public class HttpUtils {
    /**
     * 说明：根据指定URL将文件下载到指定目标位置
     *
     * @param urlPath
     *            下载路径
     * @param downloadDir
     *            文件存放目录
     * @return 返回下载文件
     */
    @SuppressWarnings("finally")
    public static File downloadFile(String urlPath, String downloadDir) throws Exception {
        File file = null;
        //中午转unicode
        String fileFullName = urlPath.substring(urlPath.lastIndexOf("/") + 1 ,urlPath.length());
        String parentPath = urlPath.substring(0 ,urlPath.lastIndexOf("/") + 1);
        // 统一资源
        URL url = new URL(parentPath + URLEncoder.encode(fileFullName, "UTF-8"));
        // 连接类的父类，抽象类
        URLConnection urlConnection = url.openConnection();
        // http的连接类
        HttpURLConnection httpURLConnection = (HttpURLConnection) urlConnection;
        //设置超时
        httpURLConnection.setConnectTimeout(1000*5);
        //设置请求方式，默认是GET
        httpURLConnection.setRequestMethod("GET");
        // 设置字符编码
        httpURLConnection.setRequestProperty("Charset", "UTF-8");
        // 打开到此 URL引用的资源的通信链接（如果尚未建立这样的连接）。
        httpURLConnection.connect();
        // 文件大小
        int fileLength = httpURLConnection.getContentLength();

        // 控制台打印文件大小
        System.out.println("您要下载的文件大小为:" + fileLength / (1024 * 1024) + "MB");

        // 建立链接从请求中获取数据
        URLConnection con = url.openConnection();
        BufferedInputStream bin = new BufferedInputStream(httpURLConnection.getInputStream());
        // 指定存放位置(有需求可以自定义)
        String fileName = fileFullName.substring(14,fileFullName.length());
        String path = downloadDir + File.separatorChar + fileName;
        file = new File(path);
        // 校验文件夹目录是否存在，不存在就创建一个目录
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }

        OutputStream out = new FileOutputStream(file);
        int size = 0;
        int len = 0;
        byte[] buf = new byte[2048];
        while ((size = bin.read(buf)) != -1) {
            len += size;
            out.write(buf, 0, size);
            // 控制台打印文件下载的百分比情况
            System.out.println("下载了-------> " + len * 100 / fileLength + "%\n");
        }
        // 关闭资源
        bin.close();
        out.close();
        System.out.println("文件下载成功！");

        return file;
//        try {
//        } catch (Exception e) {
//            System.out.println("文件下载失败！");
//        } finally {
//        }

    }

//    /**
//     * 测试
//     *
//     * @param args
//     */
//    public static void main(String[] args) {
//        // 指定资源地址，下载文件测试
//        downloadFile("http://count.liqucn.com/d.php?id=22709&urlos=android&from_type=web", "B:/myFiles/");
//
//    }
}
