package com.pjb.springbootjjwt.utils;


/* Written by Patrick Hoonhout */

import com.teamcenter.clientx.AppXSession;
import com.teamcenter.schemas.soa._2006_03.exceptions.ServiceException;
import com.teamcenter.services.strong.core.DataManagementService;
import com.teamcenter.services.strong.core.SessionService;
import com.teamcenter.services.strong.workflow.WorkflowService;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.AssignmentLists;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.AttachmentInfo;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.ContextData;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.CreateSignoffInfo;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.CreateSignoffs;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.InstanceInfo;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.Tasks;
import com.teamcenter.services.strong.workflow._2008_06.Workflow.Templates;
import com.teamcenter.soa.client.model.ModelObject;
import com.teamcenter.soa.client.model.ServiceData;
import com.teamcenter.soa.client.model.strong.EPMAssignmentList;
import com.teamcenter.soa.client.model.strong.EPMJob;
import com.teamcenter.soa.client.model.strong.EPMPerformSignoffTask;
import com.teamcenter.soa.client.model.strong.EPMResourceList;
import com.teamcenter.soa.client.model.strong.EPMReviewTask;
import com.teamcenter.soa.client.model.strong.EPMSelectSignoffTask;
import com.teamcenter.soa.client.model.strong.EPMSignoffProfile;
import com.teamcenter.soa.client.model.strong.EPMTask;
import com.teamcenter.soa.client.model.strong.EPMTaskTemplate;
import com.teamcenter.soa.client.model.strong.GroupMember;
import com.teamcenter.soa.client.model.strong.ItemRevision;
import com.teamcenter.soa.client.model.strong.Signoff;
import com.teamcenter.soa.client.model.strong.User;
import com.teamcenter.soa.client.model.strong.WorkspaceObject;
import com.teamcenter.soa.common.ObjectPropertyPolicy;
import com.teamcenter.soa.common.PolicyProperty;
import com.teamcenter.soa.common.PolicyType;
import com.teamcenter.soa.exceptions.NotLoadedException;

public class WorkFlowUtil {

//        private ItemRevision itemRev;
//        private static GroupMember grpMember;
//        private EPMJob job = null;
//        private User user;

//        private String ItemRevUID = "wNA9ziMB4IElDC";
//        private String GroupMemberUID = "A9F5GoDs4IElDC";

    // SoaEPMAction
    public static final String EPM_assign_action = "SOA_EPM_assign_action";
    public static final String	EPM_start_action = "SOA_EPM_start_action";
    public static final String	EPM_complete_action = "SOA_EPM_complete_action";
    public static final String EPM_skip_action = "SOA_EPM_skip_action";
    public static final String EPM_suspend_action = "SOA_EPM_suspend_action";
    public static final String EPM_resume_action = "SOA_EPM_resume_action";
    public static final String EPM_undo_action = "SOA_EPM_undo_action";
    public static final String EPM_abort_action = "SOA_EPM_abort_action";
    public static final String EPM_perform_action = "SOA_EPM_perform_action";
    public static final String EPM_add_attachment_action = "SOA_EPM_add_attachment_action";
    public static final String EPM_remove_attachment_action = "SOA_EPM_remove_attachment_action";
    public static final String EPM_approve_action = "SOA_EPM_approve_action";
    public static final String EPM_reject_action = "SOA_EPM_reject_action";
    public static final String EPM_promote_action = "SOA_EPM_promote_action";
    public static final String EPM_demote_action = "SOA_EPM_demote_action";
    public static final String EPM_refuse_action = "SOA_EPM_refuse_action";
    public static final String EPM_assign_approver_action = "SOA_EPM_assign_approver_action";
    public static final String EPM_notify_action = "SOA_EPM_notify_action";
    public static final String EPM_no_action = "SOA_EPM_no_action";
    public static final String EPM_fail_action	= "SOA_EPM_fail_action";

    // SoaEPMSupportingValues
    public static final String EPM_no_decision = "SOA_EPM_no_decision";
    public static final String EPM_approve = "SOA_EPM_approve";
    public static final String EPM_reject = "SOA_EPM_reject";
    public static final String EPM_unset = "SOA_EPM_unset";
    public static final String EPM_completed = "SOA_EPM_completed";
    public static final String EPM_unable_to_complete = "SOA_EPM_unable_to_complete";
    public static final String EPM_true = "SOA_EPM_true";
    public static final String EPM_false = "SOA_EPM_false";
    public static final String EPM_no_error = "SOA_EPM_no_error";

    // SignoffAction
    public static final String EPM_ACTION_UNDEFINED = "SOA_EPM_ACTION_UNDEFINED";
    public static final String EPM_Review = "SOA_EPM_Review";
    public static final String EPM_Acknowledge = "SOA_EPM_Acknowledge";
    public static final String EPM_Notify = "SOA_EPM_Notify";

    // OriginType
    public static final String EPM_Origin_Undefined = "SOA_EPM_ORIGIN_UNDEFINED";
    public static final String EPM_Signeoff_Origin_Profile = "SOA_EPM_SIGNOFF_ORIGIN_PROFILE";
    public static final String EPM_Signoff_Origin_AddressList = "SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST";

    public WorkFlowUtil() {
//        setObjectPolicy();
    }
    // This action uses OOTB Workflow "Required Signoff"
    //修改流程节点用户并且执行流程(仅限审核节点选择用户节点未完成的情况下)
    public static boolean performAction(AppXSession session, ItemRevision itemRev, String nodeName, String workflowName,
                                        GroupMember grpMember) throws Exception {
        TCUtils.setModelObjectValue(session, itemRev,new String[]{"process_stage_list"});
        ModelObject[] stageList = itemRev.get_process_stage_list();
        if (CommonlyUtils.isEmpty(stageList)) throw new Exception("版本上的流程为空！！！");
        TCUtils.setModelObjectValue(session,stageList, new String[] {"child_tasks","object_name" });
        ModelObject[] tasks = null;
        for (ModelObject obj : stageList) {
            String name = obj.getPropertyObject("object_name").getStringValue();
            if (workflowName.equals(name)) {
                tasks = ((EPMTask)obj).get_child_tasks();
            }
        }
        if (tasks == null) throw new Exception("未找到进行中的流程!!!");
        WorkflowService workflowService = WorkflowService.getService(session.getConnection());
        for(ModelObject obj : tasks) {
            TCUtils.setModelObjectValue(session,obj,new String[]{"object_name","child_tasks"});
            String name = obj.getPropertyObject("object_name").getStringValue();
            if (!name.equals(nodeName)) continue;
            if(obj instanceof EPMReviewTask) {
                EPMTask task = (EPMTask)obj;
                Signoff signOff = null;
                ModelObject[] childTasks = task.get_child_tasks();
                TCUtils.setModelObjectValue(session,childTasks,new String[]{"signoff_profiles"});
                for(ModelObject child : childTasks) {
                    if(child instanceof EPMSelectSignoffTask) {
                        EPMTask childTask = (EPMTask)child;
                        signOff = createProfileSignOff(workflowService,childTask,grpMember);
                        if(signOff == null)
                            throw new Exception("设置流程节点签发人员失败");
                        if(ServiceDataError(workflowService.performAction(childTask, EPM_complete_action,
                                "SOA complete.", null, EPM_completed, null))) // advance the EPMSelectSignoffTask
                            throw new Exception("执行选人节点任务出现错误");
                        else
                            return true;
                    }
//                    else if(child instanceof EPMPerformSignoffTask) {
//                        TCUtils.dataManagementService.refreshObjects(new ModelObject[]{itemRev,stageList[0],obj,child});
//                        LogUtil.info("4");
//                        EPMTask childTask = (EPMTask)child;
//                        if(signOff != null) {
//                            LogUtil.info("8");
//                            TCUtils.dataManagementService.refreshObjects(new ModelObject[]{itemRev,stageList[0],obj,child});
//                            if(ServiceDataError(TCUtils.workflowService.performAction(childTask, EPM_perform_action,
//                                    "SOA approved.", null, EPM_approve, signOff)))
//                                return false;
//                            LogUtil.info("9");
//                            TCUtils.dataManagementService.refreshObjects(new ModelObject[]{itemRev,stageList[0],obj,child});
//                            if(!ServiceDataError(TCUtils.workflowService.performAction(childTask, EPM_complete_action,
//                                    "SOA complete.", null, EPM_completed, null))) // advance the EPMPerformSignoffTask
//                                return true;
//                        }
//                    }
                }
            }
        }
        return false;
    }

    protected static Signoff createProfileSignOff(WorkflowService workflowService,EPMTask task,GroupMember grpMember)
            throws Exception {
//        ModelObject[] signoffProfiles = task.get_signoff_profiles();
//        if(signoffProfiles != null && signoffProfiles.length > 0) {
            CreateSignoffInfo profileInfo = new CreateSignoffInfo();
            CreateSignoffs signoff = new CreateSignoffs();
            profileInfo.signoffAction = EPM_Review;
//            profileInfo.origin = signoffProfiles[0];
            profileInfo.origin = null;
            profileInfo.originType = EPM_Origin_Undefined;
//            profileInfo.originType = EPM_Signeoff_Origin_Profile;
            profileInfo.signoffMember = grpMember;
            signoff.signoffInfo = new CreateSignoffInfo[]{
                    profileInfo };
            signoff.task = task;
            ServiceData sData = workflowService.addSignoffs(new CreateSignoffs[]{ signoff });
            if(!ServiceDataError(sData))
            {
                if(sData.sizeOfCreatedObjects() > 0)
                {
                    ModelObject obj = sData.getCreatedObject(0);
                    if(obj instanceof Signoff)
                        return (Signoff)obj;
                }
            }
//        }
        return null;
    }

    protected static boolean ServiceDataError(final ServiceData data) {
        if(data.sizeOfPartialErrors() > 0)
        {
            for(int i = 0; i < data.sizeOfPartialErrors(); i++)
            {
                for(String msg : data.getPartialError(i).getMessages())
                    System.out.println(msg);
            }
            return true;
        }
        return false;
    }

    protected void setObjectPolicy() {
        SessionService session = SessionService.getService(AppXSession.getConnection());
        ObjectPropertyPolicy policy = new ObjectPropertyPolicy();
        PolicyType itemRevType = new PolicyType("ItemRevision");
        PolicyProperty property = new PolicyProperty("process_stage_list");
        property.setModifier(PolicyProperty.WITH_PROPERTIES, true);
        itemRevType.addProperty(property);
        policy.addType(itemRevType);
        PolicyType assignmentListType = new PolicyType("EPMAssignmentList");
        property = new PolicyProperty("resoruces");
        property.setModifier(PolicyProperty.WITH_PROPERTIES, true);
        assignmentListType.addProperty(property);
        policy.addType(assignmentListType);
        policy.addType("EPMTask", new String[]{ "object_type", "object_name", "task_type",
                "task_template", "parent_process" });
        policy.addType("EPMJob", new String[]{ "root_task" });
        policy.addType("EPMTaskTemplate", new String[]{ "object_type", "object_name"});
        policy.addType("EPMAssignmentList", new String[]{ "resoruces"});
        policy.addType("EPMResourceList", new String[]{ "profiles" });
        // policy.addType("WorkspaceObject", new String[]{"object_name"});
        session.setObjectPropertyPolicy(policy);
    }
}
